;Colour bars program
;-------------------
;Binary created using DAsm 2.12 running on an Amiga.

   PROCESSOR   6502

MAXSPRITES EQU #$40

VBPASS EQU  #$0000
ORIGIN EQU  #$0001
SPRITENUM EQU #$0002
SPINSPEED EQU #$0003

TEMP1 EQU   #$0010
TEMP2 EQU   #$0011

CONTROLLER1 EQU #$0020

MULT1 EQU   #$00FC      ; For use with Multiply Subroutine
MULT2 EQU   #$00FD      ;
PRODL EQU   #$00FE      ;
PRODH EQU   #$00FF      ;

ANGLETBL EQU   #$0600   ;Table in RAM for Angle values
QUADTBL  EQU   #$0640   ;Table in RAM for Quadrant values.  Wasteful, I know.
RADIUSTBL EQU  #$0680   ;Table in RAM for Radius values.

SPRITERAM EQU  #$0700

   ORG   $E000    ;16Kb PRG-ROM, 8Kb CHR-ROM

Reset_Routine  SUBROUTINE
   cld         ;Clear decimal flag
   sei         ;Disable interrupts
.WaitV   lda $2002
   bpl .WaitV     ;Wait for vertical blanking interval
   ldx #$00
   stx $2000
   stx $2001      ;Screen display off, amongst other things
   dex
   txs         ;Top of stack at $1FF

;Clear (most of) the NES' WRAM. This routine is ripped from "Duck Hunt" - I should probably clear all
;$800 bytes.
   ldy #$06    ;To clear 7 x $100 bytes, from $000 to $6FF?
   sty $01        ;Store count value in $01
   ldy #$00
   sty $00
   lda #$00

.Clear   sta ($00),y    ;Clear $100 bytes
   dey
   bne .Clear

   dec $01        ;Decrement "banks" left counter
   bpl .Clear     ;Do next if >= 0


   lda   #$20
   sta   $2006
   lda   #$00
   sta   $2006

   ldx   #$00
   ldy   #$10
.ClearPPU sta $2007
   dex
   bne   .ClearPPU
   dey
   bne   .ClearPPU

   lda   #$00
   sta   $2003
   tay
.ClearSpr sta   $2004
   dey
   bne   .ClearSpr

   ldx   #$00
.CopyTables             ;Copy sprite info tables into RAM.
   lda   .Tables,X
   sta   ANGLETBL,X
   inx
   cpx   #$C0
   bne   .CopyTables

   ldx   #$00
.CopySprites
   lda   .Sprites,X
   sta   SPRITERAM,X
   inx
   bne   .CopySprites


;********* Initialize Palette to specified colour ********

   ldx   #$3F
   stx   $2006
   ldx   #$00
   stx   $2006

   ldx   #$0D     ;Colour Value (Black)
   ldy   #$10     ;Clear BG palette.
.InitPal stx $2007
   dey
   bne   .InitPal
   ldx   #$00
   ldy   #$10
.WritePal
   lda   .Palette,X
   sta   $2007
   inx
   dey
   bne   .WritePal


;*********************************************************

   lda   #$00
   sta   VBPASS
   lda   #$80
   sta   ORIGIN
   lda   #$01
   sta   SPINSPEED

;Enable vblank interrupts, etc.
   lda   #%10000000
   sta   $2000
   lda   #%00011110        ;Screen on, sprites on, show leftmost 8 pixels, colour
   sta   $2001
;  cli            ;Enable interrupts(?)

;Now just loop forever?
.Loop
   lda   VBPASS
   beq   .Loop
   eor   #$01
   sta   VBPASS      ;Make VBPass 0 again.

   ldx   #$00
.SprPolLoop

;-------- Calculate Polar value and save to RAM ------
;-----------------------------------------------------

   lda   ANGLETBL,X     ;Load Angle...
   sta   TEMP2          ;Save it for storage...
   lda   QUADTBL,X      ;Find out the quadrant for X
   stx   TEMP1
   and   #$01           ;Is it in quadrant 2 or 4?
   beq   .NotOddQuadX   ;If no, then skip subtraction
   lda   #$FF
   sec
   sbc   TEMP2          ;Do two's complement of Angle...
   sta   TEMP2          ;Save it for storage

.NotOddQuadX
   ldx   TEMP2          ;Transfer angle into X

   lda   .PolarTab,X    ;Pick up SINE table (from angle in index).
   ldx   TEMP1          ;Restore original X-index.
   sta   MULT1
   lda   RADIUSTBL,X    ;Pick up radius of current sprite.
   sta   MULT2
   jsr   Multiply       ;Multiply direction by radius.
   lda   PRODH
   sta   TEMP2          ;Get product and store it for manipulation.
   lda   QUADTBL,X      ;Get quadrant information again.

   and   #$02           ;Is it in quadrant 3 or 4?
   beq   .NotNegQuadX

   lda   ORIGIN
   sec
   sbc   TEMP2          ;Subtract position from origin
   sta   TEMP2
   jmp   .SaveXPos

.NotNegQuadX
   lda   ORIGIN
   clc
   adc   TEMP2          ;Add position to origin
   sta   TEMP2

.SaveXPos
   txa                  ;Multiply X-index by 4 and add 3, to get at Sprite X-pos.
   asl                  ;
   asl                  ;
   clc                  ;
   adc   #$03           ;
   tax
   lda   TEMP2
   sta   SPRITERAM,X    ;Save to whatever SPR-RAM is...
   ldx   TEMP1          ;Restore X
;----------------------------------------------
;------ Now for Y -----------------------------
;----------------------------------------------
   lda   ANGLETBL,X     ;Load Angle...
   sta   TEMP2          ;Save it for storage...
   lda   QUADTBL,X      ;Find out the quadrant for Y
   stx   TEMP1
   and   #$01           ;Is it in quadrant 1 or 3?
   bne   .NotEvenQuadY  ;If no, then skip subtraction
   lda   #$FF
   sec
   sbc   TEMP2          ;Do two's complement of Angle...
   sta   TEMP2          ;Save it for storage

.NotEvenQuadY
   ldx   TEMP2          ;Transfer angle into X

   lda   .PolarTab,X    ;Pick up COS table (from angle in index).
   ldx   TEMP1          ;Restore original X-index.
   sta   MULT1
   lda   RADIUSTBL,X    ;Pick up radius of current sprite.
   sta   MULT2
   jsr   Multiply       ;Multiply direction by radius.
   lda   PRODH
   sta   TEMP2          ;Get product and store it for manipulation.
   lda   QUADTBL,X      ;Get quadrant information again.

   cmp   #$00        ;Is it in quadrant 2 or 3?  Hoo boy...
   beq   .NotNegQuadY
   cmp   #$03
   beq   .NotNegQuadY

   lda   ORIGIN
   sec
   sbc   TEMP2          ;Subtract position from origin
   sta   TEMP2
   jmp   .SaveYPos

.NotNegQuadY
   lda   ORIGIN
   clc
   adc   TEMP2          ;Add position to origin
   sta   TEMP2

.SaveYPos
   txa                  ;Multiply X-index by 4  to get at Sprite Y-pos.
   asl                  ;
   asl                  ;
   tax
   lda   TEMP2
   sta   SPRITERAM,X    ;Save to whatever SPR-RAM is...
   ldx   TEMP1          ;Restore X-index

;==============================================


   inx
   cpx   #MAXSPRITES
   beq   .FinishedForNow
   jmp   .SprPolLoop

.FinishedForNow

   ldx   #$00
.IncAngle
   lda   ANGLETBL,X
   clc
   adc   #$01
   sta   ANGLETBL,X
   bcc   .NoAngleOverflow
   inc   QUADTBL,X
   lda   QUADTBL,X
   cmp   #$04
   bne   .NoAngleOverflow
   lda   #$00
   sta   QUADTBL,X
.NoAngleOverflow
   inx
   cpx   #MAXSPRITES
   bne   .IncAngle

   jmp   .Loop

.Tables INCLUDE PolarTables.ASM
.Sprites INCLUDE PolarSPR.ASM
.PolarTab INCLUDE Polartab.TXT

.Palette
   dc.b  #$0D,#$2D,#$10,#$20,#$0D,#$16,#$27,#$38
   dc.b  #$0D,#$2A,#$2B,#$21,#$0D,#$12,#$13,#$24

;***********************************************************************

Multiply SUBROUTINE
;----------------------------------------------------
; Function: Multiplies two 8-bit unsigned numbers
; Inputs:   MULT1 holds one number
;           MULT2 holds other number
; Outputs:  PRODL,PRODH hold 16-bit product
;----------------------------------------------------
   pha
   txa
   pha

   lda   #$00
   sta   PRODL    ;Clear low byte.
   ldx   #$08     ;Do for 8 bits.
.MultLoop
   lsr   MULT1    ;Shift and add if carry is 1.
   bcc   .MultNoAdd
   clc
   adc   MULT2
.MultNoAdd
   ror            ;a.  Rotate carry out of a and into product.
   ror   PRODL
   dex
   bne   .MultLoop
   sta   PRODH

   pla
   tax
   pla
   rts




;******************************************************

NMI_Routine SUBROUTINE
   pha

   lda   #%00010110
   sta   $2001

   lda   #$01
   sta   VBPASS

;.ReadController
;   ldx   #$01
;   stx   $4016
;   dex
;   stx   $4016
;   ldy   #$08     ;Read 8 bits from $4016
;   lda   #$00
;   sta   CONTROLLER1
;.ReadCont1
;   lda   $4016
;   ror
;   lda   CONTROLLER1
;   rol
;   sta   CONTROLLER1
;   dey
;   bne   .ReadCont1
;   sta   CONTROLLER1
;
;   lda   CONTROLLER1
;   and   #%00001000
;   beq   .NotUp
;   dec   RADIUS
;
;.NotUp
;   lda   CONTROLLER1
;   and   #%00000100
;   beq   .NotDown
;   inc   RADIUS
;
;.NotDown
;   lda   CONTROLLER1
;   and   #%00000010
;   beq   .NotLeft
;   lda   SPINSPEED
;   beq   .NotLeft
;   dec   SPINSPEED
;
;.NotLeft
;   lda   CONTROLLER1
;   and   #$01
;   beq   .NotRight
;   lda   SPINSPEED
;   cmp   #$80
;   beq   .NotRight
;   inc   SPINSPEED
;.NotRight


   lda   #$07
   sta   $4014       ;Copy $700+ to sprite-ram.

   pla
   rti

IRQ_Routine       ;Dummy label
   rti


;That's all the code. Now we just need to set the vector table approriately.

   ORG   $FFFA,0
   dc.w  NMI_Routine
   dc.w  Reset_Routine
   dc.w  IRQ_Routine    ;Not used, just points to RTI


;The end.
